#include <mega16.h>
#include <delay.h>
#include <stdio.h>
#include <stdlib.h>

#define ADC_VREF_TYPE 0xC0

enum {STOP=0b00000000, FORWARD=0b00001010, BACK=0b00010100, LEFT=0b00001100, RIGHT=0b00010010}; 
unsigned int sharp_distance;
int m=10000;
int b=-40;
int k=0;

interrupt [ADC_INT] void adc_isr(void)
{
    unsigned int adc_data;
    adc_data=ADCW;
    if(adc_data<160)
    {
        sharp_distance=0;    
    }
    else
    {
        sharp_distance=m/(adc_data+b)-k;
    }
    delay_ms(39);
    ADCSRA|=0x40;
}

void Init_chip();
void Move(unsigned char direction, unsigned char speed, unsigned int time);
void Set_PWM(unsigned char pwm);
void Start();
void White_border();
void Black_border();
void Seek();

void main(void)
{
    Init_chip();
    Start();

    while(1)
    {
        White_border();
        //Black_border();
        Seek(); 
    }
}

void Init_chip()
{
    PORTA=0x00;
    DDRA=0x1E;
    PORTD=0x00;
    DDRD=0x10;
    TCCR1A=0x23;
    TCCR1B=0x0A;
    TCNT1H=0x00;
    TCNT1L=0x00;
    ICR1H=0x00;
    ICR1L=0x00;
    OCR1AH=0x00;
    OCR1AL=0x00;
    OCR1BH=0x00;
    OCR1BL=0x00;
    ADMUX=ADC_VREF_TYPE & 0xFF;
    ADCSRA=0x8C;
    #asm("sei")
    ADCSRA|=0x40;
}

void Set_PWM(unsigned char pwm)
{
    OCR1B=pwm*10.23;
}

void Move(unsigned char direction, unsigned char speed, unsigned int time)
{
    PORTA=direction;
    Set_PWM(speed);
    #asm("cli")
    delay_ms(time);
    #asm("sei")
}

void Start()
{
    PORTA=STOP;
    #asm("cli")
    delay_ms(5000);
    #asm("sei")
    Set_PWM(100);
    PORTA=FORWARD;   
}

void White_border()
{
    if(PINA.5==0 && PINA.6==1) //  
    {
        Move(LEFT,100,600);
    }
    else 
    if(PINA.5==1 && PINA.6==0) //  
    {
        Move(RIGHT,100,600);
    }
    else
    if(PINA.5==0 && PINA.6==0) //  
    {
        Move(BACK,100,2000);
        Move(LEFT,100,600);
    }
}

void Black_border()
{
    if(PINA.5==1 && PINA.6==0) //  
    {
        Move(LEFT,100,600);
    }
    else 
    if(PINA.5==0 && PINA.6==1) //  
    {
        Move(RIGHT,100,600);
    }
    else
    if(PINA.5==1 && PINA.6==1) //  
    {
        Move(BACK,100,2000);
        Move(LEFT,100,600);
    }
}

void Seek()
{
    if(sharp_distance!=0 && sharp_distance<=50)
    {
        Move(FORWARD,100,0);
    }
    else
    {
        Move(FORWARD,50,0);
    }
}